package com.example.notification.voice;

import java.util.List;

import org.axonframework.commandhandling.gateway.CommandGateway;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import com.example.notification.voice.command.QueuePhoneCall;

@Component
public class PhoneCallScheduler {

	private static final Logger logger = LoggerFactory.getLogger(PhoneCallScheduler.class);

	private final VoiceNotificationEntryRepository repository;
	private final CommandGateway commandGateway;
	
	public PhoneCallScheduler(VoiceNotificationEntryRepository repository, CommandGateway commandGateway) {
		this.repository = repository;
		this.commandGateway = commandGateway;
	}
	
	@Scheduled(fixedRate=60000)
	public void schedulePhoneCalls() {
		logger.info("Scheduling Phone Calls");
		
		List<VoiceNotificationEntry> notifications = repository.findAll();
		
		for (VoiceNotificationEntry notification : notifications) {
			
			commandGateway.send(new QueuePhoneCall(notification.getId()));
		}
	}
}
