package com.example.notification.voice;

import java.util.List;
import java.util.Map;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;

import org.axonframework.commandhandling.gateway.CommandGateway;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.example.notification.voice.command.CancelVoiceNotification;
import com.example.notification.voice.command.CreateVoiceNotification;

@RestController
@RequestMapping("/voice")
public class VoiceNotificationController {

	private final VoiceNotificationEntryRepository repository;
	private final CommandGateway commandGateway;
	
	public VoiceNotificationController(VoiceNotificationEntryRepository repository, CommandGateway commandGateway) {
		this.repository = repository;
		this.commandGateway = commandGateway;
	}
	
	@PostMapping(consumes="application/json")
	public CompletableFuture<String> createNotification(@RequestBody Map<String, String> request) {
		return commandGateway.send(new CreateVoiceNotification(
			UUID.randomUUID().toString(),
			request.get("enterpriseId"),
			request.get("companyId"),
			request.get("phoneNumber"), 
			Integer.parseInt(request.get("maxAttempts")),
			Integer.parseInt(request.get("minutesBetweenAttempts"))
		));
	}
	
	@GetMapping
	public List<VoiceNotificationEntry> findAll() {
		return repository.findAll();
	}
	
	@GetMapping("/{id}")
	public VoiceNotificationEntry find(@PathVariable String id) {
		return repository.findOne(id);
	}
	
	@PutMapping("/{id}")
	public CompletableFuture<Object> setStatus(@PathVariable String id, @RequestBody Map<String, String> request) {
		VoiceNotificationEntry notification = find(id);
		
		if (notification != null) {
			
		}
		return commandGateway.send(new CancelVoiceNotification(id));
	}

}
