package com.example.notification.voice;

import java.util.LinkedHashMap;

import org.axonframework.eventhandling.EventHandler;
import org.springframework.stereotype.Component;

import com.example.notification.voice.event.PhoneCallCompleted;
import com.example.notification.voice.event.PhoneCallFailed;
import com.example.notification.voice.event.PhoneCallLineWasBusy;
import com.example.notification.voice.event.PhoneCallNotAnswered;
import com.example.notification.voice.event.PhoneCallQueued;
import com.example.notification.voice.event.VoiceNotificationCanceled;
import com.example.notification.voice.event.VoiceNotificationCompleted;
import com.example.notification.voice.event.VoiceNotificationCreated;

@Component
public class VoiceNotificationEntryListener {

	private final VoiceNotificationEntryRepository repository;
	
	public VoiceNotificationEntryListener(VoiceNotificationEntryRepository repository) {
		this.repository = repository;
	}
	
	@EventHandler
	public void on(VoiceNotificationCreated event) {
		repository.save(new VoiceNotificationEntry(
			event.getId(),
			event.getEnterpriseId(),
			event.getCompanyId(),
			event.getPhoneNumber(),
			event.getMaxAttempts(),
			0,
			event.getMinutesBetweenAttempts(),
			new LinkedHashMap<Integer, PhoneCallEntry>(),
			VoiceNotificationStatus.NEW));
	}
	
	@EventHandler
	public void on(VoiceNotificationCompleted event) {
		VoiceNotificationEntry notification = repository.findOne(event.getId());
		notification.setStatus(VoiceNotificationStatus.COMPLETED);
		repository.save(notification);
	}
	
	@EventHandler
	public void on(VoiceNotificationCanceled event) {
		VoiceNotificationEntry notification = repository.findOne(event.getId());
		notification.setStatus(VoiceNotificationStatus.CANCELED);
		repository.save(notification);
	}

	@EventHandler
	public void on(PhoneCallQueued event) {
		VoiceNotificationEntry notification = repository.findOne(event.getVoiceNotificationId());
		notification.getPhoneCalls().put(event.getAttempt(), new PhoneCallEntry());
		repository.save(notification);
	}

	@EventHandler
	public void on(PhoneCallCompleted event) {
		VoiceNotificationEntry notification = repository.findOne(event.getVoiceNotificationId());
		repository.save(notification);
	}

	@EventHandler
	public void on(PhoneCallFailed event) {
		VoiceNotificationEntry notification = repository.findOne(event.getVoiceNotificationId());
		repository.save(notification);
	}

	@EventHandler
	public void on(PhoneCallLineWasBusy event) {
		VoiceNotificationEntry notification = repository.findOne(event.getVoiceNotificationId());
		repository.save(notification);
	}

	@EventHandler
	public void on(PhoneCallNotAnswered event) {
		VoiceNotificationEntry notification = repository.findOne(event.getVoiceNotificationId());
		repository.save(notification);
	}
}
