package com.example.notification.voice.command;

import static org.axonframework.commandhandling.model.AggregateLifecycle.apply;

import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Map;

import org.axonframework.commandhandling.CommandHandler;
import org.axonframework.commandhandling.model.AggregateIdentifier;
import org.axonframework.eventsourcing.EventSourcingHandler;
import org.axonframework.spring.stereotype.Aggregate;

import com.example.notification.voice.PhoneCallStatus;
import com.example.notification.voice.VoiceNotificationStatus;
import com.example.notification.voice.event.PhoneCallCompleted;
import com.example.notification.voice.event.PhoneCallFailed;
import com.example.notification.voice.event.PhoneCallLineWasBusy;
import com.example.notification.voice.event.PhoneCallNotAnswered;
import com.example.notification.voice.event.PhoneCallQueued;
import com.example.notification.voice.event.VoiceNotificationCanceled;
import com.example.notification.voice.event.VoiceNotificationCreated;

import lombok.Data;
import lombok.NoArgsConstructor;

@Data
@Aggregate
@NoArgsConstructor
public class VoiceNotification {
	
	@AggregateIdentifier
	private String id;

	private String
		enterpriseId,
		companyId,
		phoneNumber;
		
	private Date
		completed;
	
	private int
		maxAttempts,
		attempts,
		minutesBetweenAttempts;
	
	private final Map<Integer, PhoneCall>
		phoneCalls = new LinkedHashMap<>();
	
	private VoiceNotificationStatus
		status;
	
	@CommandHandler
	public VoiceNotification(CreateVoiceNotification command) {
		apply(new VoiceNotificationCreated(
			command.getId(),
			command.getEnterpriseId(),
			command.getCompanyId(),
			command.getPhoneNumber(),
			command.getMaxAttempts(),
			command.getMinutesBetweenAttempts()));
	}
	
	@CommandHandler
	public void handle(CancelVoiceNotification command) {
		if (getStatus() == VoiceNotificationStatus.NEW || getStatus() == VoiceNotificationStatus.IN_PROGRESS) {
			apply(new VoiceNotificationCanceled(command.getId()));
		}
	}
	
	@EventSourcingHandler
	public void on(VoiceNotificationCreated event) {
		setId(event.getId());
		setEnterpriseId(event.getEnterpriseId());
		setCompanyId(event.getCompanyId());
		setPhoneNumber(event.getPhoneNumber());
		setMaxAttempts(event.getMaxAttempts());
		setMinutesBetweenAttempts(event.getMinutesBetweenAttempts());
		setAttempts(0);
		setStatus(VoiceNotificationStatus.NEW);
	}
	
	@EventSourcingHandler
	public void on(VoiceNotificationCanceled event) {
		setStatus(VoiceNotificationStatus.CANCELED);
	}
	
	@CommandHandler
	public void handle(QueuePhoneCall command) {
		if (getAttempts() < getMaxAttempts()) {
			apply(new PhoneCallQueued(getId(), getAttempts() + 1));
		}
	}
	
	@EventSourcingHandler
	public void on(PhoneCallQueued event) {
		getPhoneCalls().put(event.getAttempt(), new PhoneCall(null, PhoneCallStatus.QUEUED));
	}
	
	@EventSourcingHandler
	public void on(PhoneCallLineWasBusy event) {
		attempts++;
		getPhoneCalls().get(event.getAttempt()).setStatus(PhoneCallStatus.BUSY);
	}
	
	@EventSourcingHandler
	public void on(PhoneCallFailed event) {
		attempts++;
		getPhoneCalls().get(event.getAttempt()).setStatus(PhoneCallStatus.FAILED);
	}
	
	@EventSourcingHandler
	public void on(PhoneCallNotAnswered event) {
		attempts++;
		getPhoneCalls().get(event.getAttempt()).setStatus(PhoneCallStatus.NO_ANSWER);
	}
	
	@EventSourcingHandler
	public void on(PhoneCallCompleted event) {
		attempts++;
		getPhoneCalls().get(event.getAttempt()).setStatus(PhoneCallStatus.COMPLETED);
	}
}
