package com.example.notification.voice;

import java.util.UUID;

import org.axonframework.test.aggregate.AggregateTestFixture;
import org.junit.Before;
import org.junit.Test;

import com.example.notification.voice.command.CancelVoiceNotification;
import com.example.notification.voice.command.CreateVoiceNotification;
import com.example.notification.voice.command.QueuePhoneCall;
import com.example.notification.voice.command.VoiceNotification;
import com.example.notification.voice.event.PhoneCallQueued;
import com.example.notification.voice.event.VoiceNotificationCanceled;
import com.example.notification.voice.event.VoiceNotificationCreated;

public class VoiceNotificationTest {

	private AggregateTestFixture<VoiceNotification> fixture;

	@Before
	public void setUp() throws Exception {
		fixture = new AggregateTestFixture<VoiceNotification>(VoiceNotification.class);
	}
	
	@Test
	public void testCreateVoiceNotification() {
		String id = UUID.randomUUID().toString();
		
		fixture
			.givenNoPriorActivity()
			.when(new CreateVoiceNotification(id, "1", "1", "", 5, 10))
			.expectEvents(new VoiceNotificationCreated(id, "1", "1", "", 5, 10));
	}
	
	@Test
	public void testCancelVoiceNotification() {
		String id = UUID.randomUUID().toString();
		
		fixture
			.given(new VoiceNotificationCreated(id, "1", "1", "", 5, 10))
			.when(new CancelVoiceNotification(id))
			.expectEvents(new VoiceNotificationCanceled(id));
	}
	
	@Test
	public void testQueuePhoneCall() {
		String id = UUID.randomUUID().toString();
		
		fixture
			.given(new VoiceNotificationCreated(id, "1", "1", "", 5, 10), new QueuePhoneCall(id))
			.when(new QueuePhoneCall(id))
			.expectEvents(new PhoneCallQueued(id, 2));
	}
}
